==> Epics8.80.tar.gz: Epics8.80/prog/Elemag/epStern.f <==
c     ****************************************************************
c     *                                                              *
c     * epShtern:  to compute coefficient which appear in sternheimers *
c     *          dE/dx expression.
c     *        tested                                                 *
c
c /usage/
c
c      See Sternheimer, p.r. b vol.3(1971)3681; bef v.8.80
c        
c    This is updated in v.8.80 
c     
c
c
      subroutine epStern(w0in, media)
      implicit none
#include "Zmedia.h"
#include "Zcode.h"
#include "Zptcl.h"
#include "Zmass.h"

      real*8  w0in  !  input. kinetic energy of knock-on electrons > w0in
                    !         is not taken into account as energy loss( GeV).

      record  /epmedia/media  ! input. 
c                             media.I, media.Z,media.A,
c                             media.rho, media.gasF
c                             epExpot must have been  called.
                             ! output.  media.sh
c                     

      record  /ptcl/electron
c
      record  /sternh/sh    !  Sterhnheimer's consts is stored here  tempo.
c                           and copied to media.sh later
c
      real*8  hnp, cb, ep
c

      ep = media.I * 1.d9  ! ep in  eV.
      hnp = media.wp*1.d9  ! plasma energy in eV
c             mdeia.wp in GeV should have been fixed
c             in epGetEffZA
c           next two  the same
c      sh.a = 0.1536* (media.Z/media.A) ! 0.1536 was bef. v8.80
      media.sh.a = 0.153536* media.ZbyAeff
c      sh.b is defined diffrently from v8.80;
c      it comes from a part of the term  ln(2mg^ b^2Tm/I^2)
c      = ln(m^2/I^2) + ln(2*g^2b^2Tm/m) 
c      = 2ln(m/I) + ln(2*g^2b^2Tm/m) 
c      = -2ln(I/m)+ ln(2*g^2b^2Tm/m) 
c        (g= gamma , b=beta, m=massele, Tm=max recoil  kinetic E)
c            
c      the first term is sh.b; second term is taken into account
c      at dE/dx calculation.  (for e-; Tm/m= (g-1)/2;
c      for e+; Tm/m = g-1 )
      media.sh.b = 2*log(masele*1.d9/ep)
c
      sh.c = -2*log(ep/hnp) - 1.
c         Cbar is -sh.c 
      cb =- sh.c
      if(media.gasF .eq. 0) then
c               solid /liquid
         if(ep  .lt. 100.) then
            if(cb .lt. 3.681) then
               sh.x0=0.2
            else
               sh.x0=0.326*cb -1.0
            endif
            sh.x1=2.0
         else
            if(cb .lt. 5.215) then ! bef. v8.80 5.211
               sh.x0=0.2
            else
               sh.x0=0.326*cb-1.5
            endif
            sh.x1=3.0
         endif
c            exception
         if( media.name .eq. "LiqH2" ) then
            sh.k = 5.949
            sh.x0 = 0.425
            sh.x1 = 2.0
         else
            sh.k = 3.
         endif
      else
c             gas 1 atm 0degree ; lot of diff. from versios  bef. 8.80
         if(cb .lt. 12.25) then
            sh.x1=4.
            if(cb .ge. 11.5) then
               sh.x0 = 2.0
            elseif(cb .ge. 11.0) then
               sh.x0 = 1.9
            elseif(cb .ge. 10.5) then
               sh.x0 = 1.8
            elseif(cb .ge. 10.0) then
               sh.x0 = 1.7
            else
               sh.x0 = 1.6
            endif
         else
            sh.x1=5.0
            if(cb .lt. 13.804) then            
               sh.x0 = 2.0
            else
               sh.x0=0.326*cb - 2.5
            endif
         endif
c          exceptions
         if( media.name .eq. "H2") then
            sh.k = 4.754
            sh.x0 = 1.837
            sh.x1 = 3.0
         elseif( media.name .eq. "Hegas") then
            sh.k = 3.297
            sh.x0 = 2.191
            sh.x1 = 3.0
         else
            sh.k = 3.   
         endif
      endif
c      media.sh.xa=cb/4.60517          !  2ln(10)
c      sh.sa = (cb-4.60517X0)/((x1-X0)**k
      sh.sa=(cb-4.60517*sh.x0)/(sh.x1-sh.x0)**sh.k
c       some additional const relating to restriced energy loss
      media.sh.tcut = w0in        ! in GeV 
      media.sh.w0 = w0in * 1000.  ! in MeV unit. 
      media.sh.wlg0 = log(media.sh.w0) 


      if( media.format .eq. 2) then
         write(0,*)
     *   "(Sternheimer's) Density effect correction consts. "
         write(0,*)
     *    "Normally diff. between the  input values and ones computed"
         write(0,*) " is rather large, but resultant dE/dx is simlar."
         write(0,*)
     *    " We use input values.  However, check if diff. is large"
c         write(0,*) "If diff < 1%, the values will  not be  printed"
         call epchkSternConst("c",  sh.c, media.sh.c)
         call epchkSternConst("x0", sh.x0, media.sh.x0)
         call epchkSternConst("x1", sh.x1, media.sh.x1)
         call epchkSternConst("a",  sh.sa, media.sh.sa)
         call epchkSternConst("k",  sh.k, media.sh.k)
         call epchkSternConst("delta0",  0.d0, media.sh.delta0)         
      else
         media.sh.c = sh.c
         media.sh.x0 = sh.x0
         media.sh.x1 = sh.x1
         media.sh.sa = sh.sa
         media.sh.k = sh.k
         media.sh.delta0 = 0.
      endif
c         next one is used only when EdepdEdx = f.
c         for compararison with analytical calc.
c         In that case, w0in must be as large as
c         incident particle energy.
      call cmkptc(kelec, regptcl, -1, electron)
      electron.fm.p(4)= 3.162 * masele  
      call epdedxe(media, electron, media.dEdxatp3m)
      end
c         ***************************
      subroutine  epchkSternConst(name, cmp, inp)
      implicit none
c         check Sternheimer's const computed vs input
c
      character*(*)  name  ! input. variable name
      real*8  cmp  ! input.  computed value in epStern
      real*8  inp  ! input.  corresponding value given by input (fmt=2)
                   !    inp and cmp is compared if the diff 
                   ! output.  if this is -100. cmp is put.

      if( inp .eq. -100.0) then
         inp = cmp
      else
         if(name .ne. "delta0" ) then
c            if(abs(inp-cmp)/inp .gt. 0.01) then
               write(0,*) name, ": input=", inp, " computed=",cmp
c            endif
         else
c            delta0 is always 0 in epStern but expected to be
c            a max of 0.14  for conductors
c
            if( inp .gt. 0.14) then
               write(0,*) "delta0 for ", name, " is > 0.14" 
               write(0,*) "check the value in Media file"
            endif
         endif
      endif               
      end

==> Epics9.15.tar.gz: Epics9.15/prog/Elemag/epStern.f <==
c     ****************************************************************
c     *                                                              *
c     * epShtern:  to compute coefficient which appear in sternheimers *
c     *          dE/dx expression.
c     *        tested                                                 *
c
c /usage/
c
c      See Sternheimer, p.r. b vol.3(1971)3681; bef v.8.80
c        
c    This is updated in v.8.80 
c     
c
c
      subroutine epStern(w0in, media)
      implicit none
#include "Zmedia.h"
#include "Zcode.h"
#include "Zptcl.h"
#include "Zmass.h"

      real*8  w0in  !  input. kinetic energy of knock-on electrons > w0in
                    !         is not taken into account as energy loss( GeV).

      record  /epmedia/media  ! input. 
c                             media.I, media.Z,media.A,
c                             media.rho, media.gasF
c                             epExpot must have been  called.
                             ! output.  media.sh
c                     

      record  /ptcl/electron
c
      record  /sternh/sh    !  Sterhnheimer's consts is stored here  tempo.
c                           and copied to media.sh later
c
      real*8  hnp, cb, ep, dedxfull
c

      ep = media.I * 1.d9  ! ep in  eV.
      hnp = media.wp*1.d9  ! plasma energy in eV
c             mdeia.wp in GeV should have been fixed
c             in epGetEffZA
c           next two  the same
c      sh.a = 0.1536* (media.Z/media.A) ! 0.1536 was bef. v8.80
      media.sh.a = 0.153536* media.ZbyAeff
c      sh.b is defined diffrently from v8.80;
c      it comes from a part of the term  ln(2mg^ b^2Tm/I^2)
c      = ln(m^2/I^2) + ln(2*g^2b^2Tm/m) 
c      = 2ln(m/I) + ln(2*g^2b^2Tm/m) 
c      = -2ln(I/m)+ ln(2*g^2b^2Tm/m) 
c        (g= gamma , b=beta, m=massele, Tm=max recoil  kinetic E)
c            
c      the first term is sh.b; second term is taken into account
c      at dE/dx calculation.  (for e-; Tm/m= (g-1)/2;
c      for e+; Tm/m = g-1 )
      media.sh.b = 2*log(masele*1.d9/ep)
c
      sh.c = -2*log(ep/hnp) - 1.
c         Cbar is -sh.c 
      cb =- sh.c
      if(media.gasF .eq. 0) then
c               solid /liquid
         if(ep  .lt. 100.) then
            if(cb .lt. 3.681) then
               sh.x0=0.2
            else
               sh.x0=0.326*cb -1.0
            endif
            sh.x1=2.0
         else
            if(cb .lt. 5.215) then ! bef. v8.80 5.211
               sh.x0=0.2
            else
               sh.x0=0.326*cb-1.5
            endif
            sh.x1=3.0
         endif
c            exception
         if( media.name .eq. "LiqH2" ) then
            sh.k = 5.949
            sh.x0 = 0.425
            sh.x1 = 2.0
         else
            sh.k = 3.
         endif
      else
c             gas 1 atm 0degree ; lot of diff. from versios  bef. 8.80
         if(cb .lt. 12.25) then
            sh.x1=4.
            if(cb .ge. 11.5) then
               sh.x0 = 2.0
            elseif(cb .ge. 11.0) then
               sh.x0 = 1.9
            elseif(cb .ge. 10.5) then
               sh.x0 = 1.8
            elseif(cb .ge. 10.0) then
               sh.x0 = 1.7
            else
               sh.x0 = 1.6
            endif
         else
            sh.x1=5.0
            if(cb .lt. 13.804) then            
               sh.x0 = 2.0
            else
               sh.x0=0.326*cb - 2.5
            endif
         endif
c          exceptions
         if( media.name .eq. "H2") then
            sh.k = 4.754
            sh.x0 = 1.837
            sh.x1 = 3.0
         elseif( media.name .eq. "Hegas") then
            sh.k = 3.297
            sh.x0 = 2.191
            sh.x1 = 3.0
         else
            sh.k = 3.   
         endif
      endif
c      media.sh.xa=cb/4.60517          !  2ln(10)
c      sh.sa = (cb-4.60517X0)/((x1-X0)**k
      sh.sa=(cb-4.60517*sh.x0)/(sh.x1-sh.x0)**sh.k
c       some additional const relating to restriced energy loss
      media.sh.tcut = w0in        ! in GeV 
      media.sh.w0 = w0in * 1000.  ! in MeV unit. 
      media.sh.wlg0 = log(media.sh.w0) 


      if( media.format .eq. 2) then
         write(0,*)
     *   "(Sternheimer's) Density effect correction consts. "
         write(0,*)
     *    "Normally diff. between the  input values and ones computed"
         write(0,*) " is rather large, but resultant dE/dx is simlar."
         write(0,*)
     *    " We use input values.  However, check if diff. is large"
c         write(0,*) "If diff < 1%, the values will  not be  printed"
         call epchkSternConst("c",  sh.c, media.sh.c)
         call epchkSternConst("x0", sh.x0, media.sh.x0)
         call epchkSternConst("x1", sh.x1, media.sh.x1)
         call epchkSternConst("a",  sh.sa, media.sh.sa)
         call epchkSternConst("k",  sh.k, media.sh.k)
         call epchkSternConst("delta0",  0.d0, media.sh.delta0)         
      else
         media.sh.c = sh.c
         media.sh.x0 = sh.x0
         media.sh.x1 = sh.x1
         media.sh.sa = sh.sa
         media.sh.k = sh.k
         media.sh.delta0 = 0.
      endif
c         next one is used only when EdepdEdx = f.
c         for compararison with analytical calc.
c         In that case, w0in must be as large as
c         incident particle energy.
      call cmkptc(kelec, regptcl, -1, electron)
      electron.fm.p(4)= 3.162 * masele  
      call epdedxe(media, electron, media.dEdxatp3m, dedxfull)
      end
c         ***************************
      subroutine  epchkSternConst(name, cmp, inp)
      implicit none
c         check Sternheimer's const computed vs input
c
      character*(*)  name  ! input. variable name
      real*8  cmp  ! input.  computed value in epStern
      real*8  inp  ! input.  corresponding value given by input (fmt=2)
                   !    inp and cmp is compared if the diff 
                   ! output.  if this is -100. cmp is put.

      if( inp .eq. -100.0) then
         inp = cmp
      else
         if(name .ne. "delta0" ) then
c            if(abs(inp-cmp)/inp .gt. 0.01) then
               write(0,*) name, ": input=", inp, " computed=",cmp
c            endif
         else
c            delta0 is always 0 in epStern but expected to be
c            a max of 0.14  for conductors
c
            if( inp .gt. 0.14) then
               write(0,*) "delta0 for ", name, " is > 0.14" 
               write(0,*) "check the value in Media file"
            endif
         endif
      endif               
      end
